#!/bin/sh
###############################################################################
# IBM Confidential
# OCO Source Materials
# © Copyright IBM Corporation 2015
# The source code for this program is not published or otherwise divested
# of its trade secrets, irrespective of what has been deposited with
# the U.S. Copyright office.
###############################################################################
function error {
    cd ${SPE_INSTALL_DIR}
    echo
    echo $1
    echo
    rm -rf ${WORK_DIR}
    rm -rf ${LIST_OF_DIRS}
    rm -rf ${LIST_OF_BINARY}
    rm -rf ${LIST_OF_TEXT}
    rm -rf ${LIST_OF_XML}
    rm -rf ${LIST_OF_FILES}
    exit 1
}

function syntax {
    echo "Deploys SPE patch resources"
    echo
    echo "speinstall-patch.sh -j /path/to/patch.jar"
    echo
    echo "Command line arguments:"
    echo "-j  (Required) Path to the SPE patch jar file"
    echo
}

if [ -z "$InSPEEnv" ]; then
    echo Please initialize the SPE environment before running this command.
    echo You can use spesetup.sh to initialize the environment.
else
    TIMESTAMP="D"`date +%Y%m%d`".T"`date +%H%M%S`
    PATCH_JAR=

    # Get input
    while getopts j: option
    do
        case ${option} in
            j) PATCH_JAR=${OPTARG}
               ;;
           \?) syntax
               exit
               ;;
        esac
    done

    # Validate the PATCH_JAR
    if [ -z "${PATCH_JAR}" ]
    then
        echo "Missing mandatory parameter -j"
        syntax
        exit 1
    else
        if [ ! -f "${PATCH_JAR}" ]
        then
            if [ "${PATCH_JAR}" != "nojar" ]
            then
                error "SPE patch jar file does not exist: ${PATCH_JAR}"
            fi
        else
            if [[ "${PATCH_JAR}" != *"jar" ]]
            then
                error "Invalid jar file name: ${PATCH_JAR}"
            fi
        fi
    fi

    echo PATCH_JAR=${PATCH_JAR}

    # Generate WORK_DIR name and extract jar to it
    JAR_NAME=`basename "${PATCH_JAR}"`
    echo JAR_NAME=${JAR_NAME}
    NO_JAR_NAME=${JAR_NAME%.jar}
    WORK_DIR="${SPE_INSTALL_DIR}/${NO_JAR_NAME}"
    echo WORK_DIR=${WORK_DIR}
    if [ -d ${WORK_DIR} ]
    then
        rm -rf ${WORK_DIR}
    fi

    echo
    echo "Extracting files..."
    ${JAVA_HOME}/bin/java com.ibm.spe.core.utils.ExtractJarFile -name="${PATCH_JAR}" -output="${WORK_DIR}"
    if [ $? -ne 0 ]
    then
        error "Failed to extract ${PATCH_JAR} to ${WORK_DIR}"
    fi
    cd "${WORK_DIR}"
    if [ $? -ne 0 ]
    then
        error "Failed to change directory to ${WORK_DIR}"
    fi

    # Process obsolete files
    if [ -f OBSOLETE_FILES ]
    then
        cd ${SPE_INSTALL_DIR}
        OBSOLETE_DIR=obsolete
        if [ ! -d ${OBSOLETE_DIR} ]
        then
            echo
            echo "creating ${OBSOLETE_DIR} directory"
            mkdir -p ${OBSOLETE_DIR}
            if [ $? -ne 0 ]
            then
                error "Failed to create ${OBSOLETE_DIR} directory"
            fi
        fi

        cd "${WORK_DIR}"
        echo
        echo "Moving files to the ${OBSOLETE_DIR} directory..."
        for FILE in $(<./OBSOLETE_FILES)
        do
            SOURCE_FILE="${SPE_INSTALL_DIR}/${FILE}"
            TARGET_DIR=`dirname "${SPE_INSTALL_DIR}/${OBSOLETE_DIR}/${FILE}"`
            if [ -f ${SOURCE_FILE} ]
            then
                echo "obsolete ${FILE}"
                mkdir -p ${TARGET_DIR}
                mv ${SOURCE_FILE} ${TARGET_DIR}
            fi
        done
    fi

    # Create backup directory
    cd ${SPE_INSTALL_DIR}
    BACKUP_DIR=backups/${TIMESTAMP}_PATCH
    if [ ! -f BACKUP_DIR ]
    then
        echo
        echo "creating ${BACKUP_DIR} directory"
        mkdir -p ${BACKUP_DIR}
        if [ $? -ne 0 ]
        then
            error "Failed to create ${BACKUP_DIR} directory"
        fi
    fi

    cd ${WORK_DIR}
    echo
    echo "Removing accessory files..."
    rm -rf OBSOLETE_FILES
    rm -rf CVS
    rm -rf META-INF

    echo
    echo Creating directories...
    # Get list of directories
    LIST_OF_DIRS=${NO_JAR_NAME}.dirs
    find * -type d > ../${LIST_OF_DIRS}
    # Create deploy directories
    for FILE in $(<../${LIST_OF_DIRS})
    do
        TARGET_DIR="${SPE_INSTALL_DIR}/${FILE}"
        if [ ! -d "${TARGET_DIR}" ]
        then
            mkdir -p "${TARGET_DIR}"
            if [ $? -ne 0 ]
            then
                error "Failed to create ${TARGET_DIR} directory"
            fi
        fi
    done
    # Create backup directories
    for FILE in $(<../${LIST_OF_DIRS})
    do
        TARGET_DIR="${SPE_INSTALL_DIR}/${BACKUP_DIR}/${FILE}"
        if [ ! -d "${TARGET_DIR}" ]
        then
            mkdir -p "${TARGET_DIR}"
            if [ $? -ne 0 ]
            then
                error "Failed to create ${BACKUP_DIR} directory"
            fi            
        fi
    done

    
    # Get list of files to copy
    LIST_OF_FILES=${NO_JAR_NAME}.txt
    find * -type f > ../${LIST_OF_FILES}

    echo
    echo "Deploying patch files..."
    # Copy binary files
    LIST_OF_BINARY=${NO_JAR_NAME}.binary
    egrep "\.jar|\.mmc|edisvu\.|hcsvu\.|\.txo" ../${LIST_OF_FILES} > ../${LIST_OF_BINARY}
    for FILE in $(<../${LIST_OF_BINARY})
    do
        TARGET_FILE="${SPE_INSTALL_DIR}/${FILE}"
        BACKUP_FILE="${SPE_INSTALL_DIR}/${BACKUP_DIR}/${FILE}"
        if [ -f ${TARGET_FILE} ]
        then
            echo "backing up old ${FILE}"
            mv ${TARGET_FILE} ${BACKUP_FILE}
            if [ $? -ne 0 ]
            then
                error "Failed to make a backup copy of ${FILE}"
            fi
        fi
        echo "deploying ${FILE}"
        cp -p ${FILE} ${TARGET_FILE}
        if [ $? -ne 0 ]
        then
            error "Failed to deploy ${FILE}"
        fi
    done

    # Fix line terminator, iconv (z/OS only) and copy text files
    LIST_OF_TEXT=${NO_JAR_NAME}.text
    egrep "\.txt|\.dat|\.ini|\.prop|\.sh" ../${LIST_OF_FILES} > ../${LIST_OF_TEXT}
    for FILE in $(<../${LIST_OF_TEXT})
    do
        TARGET_FILE="${SPE_INSTALL_DIR}/${FILE}"
        BACKUP_FILE="${SPE_INSTALL_DIR}/${BACKUP_DIR}/${FILE}"
        if [ -f ${TARGET_FILE} ]
        then
            echo "backing up old ${FILE}"
            mv ${TARGET_FILE} ${BACKUP_FILE}
            if [ $? -ne 0 ]
            then
                error "Failed to make a backup copy of ${FILE}"
            fi
        fi
        echo "deploying ${FILE}"
        if [ ${localOS} = "OS/390" ]
        then
            tr -d '\r' < ${FILE}  | iconv -f ISO8859-1 -t IBM-1047 > ${TARGET_FILE}
        else
            tr -d '\r' < ${FILE} > ${TARGET_FILE}
        fi
        if [ $? -ne 0 ]
        then
            error "Failed to deploy ${FILE}"
        fi
    done

    # Fix line terminator and copy xml files
    LIST_OF_XML=${NO_JAR_NAME}.xml
    egrep -v "\.jar|\.mmc|edisvu\.|hcsvu\.|\.txo|\.txt|\.dat|\.ini|\.prop|\.sh" ../${LIST_OF_FILES} > ../${LIST_OF_XML}
    for FILE in $(<../${LIST_OF_XML})
    do
        TARGET_FILE="${SPE_INSTALL_DIR}/${FILE}"
        BACKUP_FILE="${SPE_INSTALL_DIR}/${BACKUP_DIR}/${FILE}"
        if [ -f ${TARGET_FILE} ]
        then
            echo "backing up old ${FILE}"
            mv ${TARGET_FILE} ${BACKUP_FILE}
            if [ $? -ne 0 ]
            then
                error "Failed to make a backup copy of ${FILE}"
            fi
        fi
        echo "deploying ${FILE}"
        tr -d '\r' < ${FILE} > ${TARGET_FILE}
        if [ $? -ne 0 ]
        then
            error "Failed to deploy ${FILE}"
        fi
    done

    # Cleanup
    cd ${SPE_INSTALL_DIR}
    rm -rf ${WORK_DIR}
    rm -rf ${LIST_OF_DIRS}
    rm -rf ${LIST_OF_BINARY}
    rm -rf ${LIST_OF_TEXT}
    rm -rf ${LIST_OF_XML}
    rm -rf ${LIST_OF_FILES}
    echo
    echo "SPE patch resources were deployed successfully"
    echo
fi
